import unittest
import os

from pdns_exporter import exporter

EXTERNAL_SETTINGS_PATH = "/tmp/pdns-exporter/settings.conf.test"

class args:
    c = None
    v = False

class TestConfig(unittest.TestCase):
    def tearDown(self):
        """clear test data"""
        try:
            os.remove(EXTERNAL_SETTINGS_PATH)
        except: pass

        try:
            os.environ.pop("PDNSEXPORT_LOCAL_PORT")
        except: pass
        
    def test_default_config(self):
        """read default config"""
        # read config
        cfg = exporter.setup_config(args=args())

        self.assertFalse(int(cfg["logs-verbose"]))
        self.assertEqual(cfg["local-address"], "0.0.0.0")
        self.assertEqual(int(cfg["local-port"]), 9090)

    def test_overwrite_config_external(self):
        """overwrite config with external file"""
        _args = args()
        _args.c = EXTERNAL_SETTINGS_PATH

        cfg_ext = ["local-address=127.0.0.1", "local-port=8080"]
        with open(_args.c , "w") as f:
           f.write("\n".join(cfg_ext))

        # read config
        cfg = exporter.setup_config(args=_args)

        assert cfg["local-port"] == "8080"
    
    def test_overwrite_config_env(self):
        """overwrite config with env variables"""
        os.environ['PDNSEXPORT_LOCAL_PORT'] = "8888"

        # read config
        cfg = exporter.setup_config(args=args())

        # remove variable from environment for other test
        os.environ.pop("PDNSEXPORT_LOCAL_PORT")

        self.assertEqual(cfg["local-port"], "8888")




import unittest
import subprocess
import requests
import time

from pdns_exporter import exporter

class FakeResp:
    status_code = 0

class TestPackage(unittest.TestCase):
    def test_import(self):
        """test import package"""
        api_url = "http://127.0.0.1:9090/zones"
        r = FakeResp()

        cmd = ["python3", "-c", "from pdns_exporter import exporter; exporter.start_exporter();"]

        with subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT) as proc:
            for retry in range(5):
                try:
                    r = requests.get(url=api_url, auth=('changeme', 'changeme'))
                    break
                except requests.exceptions.ConnectionError:
                    time.sleep(0.5)
            proc.kill()

        print(r.status_code)
        assert r.status_code == 200



import aiohttp
import aiohttp.test_utils

from pdns_exporter import exporter
from pdns_exporter import webapp

class args:
    c = None
    v = False

BASIC_AUTH = aiohttp.BasicAuth("changeme", "changeme")

class TestWebapp(aiohttp.test_utils.AioHTTPTestCase):
    async def get_application(self):
        """get webapp"""
        cfg = exporter.setup_config(args=args())
        return webapp.setup_webapp(cfg=cfg)

    @aiohttp.test_utils.unittest_run_loop
    async def test_invalid_auth_zones(self):
        """invalid basic authentication for /zones"""
        async with self.client.get("/zones") as r:
            print(r.status)
            assert r.status == 401

    @aiohttp.test_utils.unittest_run_loop
    async def test_invalid_auth_zone(self):
        """invalid basic authentication for /zone/id"""
        async with self.client.get("/zone/1") as r:
            print(r.status)
            assert r.status == 401

    @aiohttp.test_utils.unittest_run_loop
    async def test_invalid_auth_metrics(self):
        """invalid basic authentication for /metrics"""
        async with self.client.get("/metrics") as r:
            print(r.status)
            assert r.status == 401

    @aiohttp.test_utils.unittest_run_loop
    async def test_invalid_route(self):
        """invalid route"""
        async with self.client.get("/", auth=BASIC_AUTH) as r:
            print(r.status)
            assert r.status == 404

    @aiohttp.test_utils.unittest_run_loop
    async def test_get_all_zones(self):
        """get all zones"""
        async with self.client.get("/zones", auth=BASIC_AUTH) as r:
            print(r.status)
            assert r.status == 200
            zones = await r.json()
            assert len(zones) > 0

    @aiohttp.test_utils.unittest_run_loop
    async def test_get_one_zone(self):
        """get zone in bind format"""
        async with self.client.get("/zone/1", auth=BASIC_AUTH) as r:
            print(r.status)
            assert r.status == 200
            plaintext = await r.text()
            assert "$ORIGIN ." in plaintext

            with open("/tmp/db.zone.test", "w") as f:
                f.write(plaintext)

    @aiohttp.test_utils.unittest_run_loop
    async def test_get_metrics(self):
        """get metrics"""
        async with self.client.get("/metrics", auth=BASIC_AUTH) as r:
            print(r.status)
            assert r.status == 200
            plaintext = await r.text()
            assert "# HELP pdnsexporter_zones" in plaintext
            assert "# HELP pdnsexporter_records" in plaintext